<?php
/**
 * Favatars
 *
 * A CakePHP-adapted version of the Wordpress Favatars plugin for fetching favicons from a URL.
 * Original version by Jeff Minard
 * Stripped down into a CakePHP-friendly class by Ben Barnett.
 *
 * @package Favatars
 * @author Jeff Minard (jrm.cc), Modified by Ben Barnett (menial.co.uk)
 * @version 2
 * @link http://dev.wp-plugins.org/wiki/favatars Original Source
 * @link http://menial.co.uk/ This version
 **/
class FavatarsController extends AppController
{
	var $name = 'Favatar';
	var $pageTitle = 'Favatar';
	var $uses = null;
	var $components = null;
	var $helpers = null;
	var $default_url = null; // Set in __construct. MUST BE A VALID favicon URL!
	
	
	function __construct()
	{
		parent::__construct();
		vendor('httprequest');
		$this->default_url = WWW_ROOT.IMAGES_URL.'globe.ico';
	}
	
	
	/**
	 * Outputs a favicon based on a GET parameter
	 * 
	 * Will attempt to locate a favicon for an address supplied in $_GET['link'].
	 * Found favicons are cached for 1 week.
	 * Favicons not found return a default icon. This is not cached.
	 */
	function favatar() 
	{
		$url = urldecode($_GET['link']);
		$url = (!empty($url)) ? $url : 'localhost';
		$url_tmp = parse_url($url);
		if (!isset($url_tmp['scheme'])) {
			$url = 'http://'.$url;
		}
		$file = cache('favicons'.DS.$this->url2filename($url).'.ico', null);
		if ($file) {
			$this->output($file);
			exit();
		}
		
		// start by fetching the contents of the URL they left...
		if($html = @file_get_contents($url)) 
		{
			if (preg_match('/<link[^>]+rel="(?:shortcut )?icon"[^>]+?href="([^"]+?)"/si', $html, $matches)) 
			{
				// Attempt to grab a favicon link from their webpage url

				$linkUrl = html_entity_decode($matches[1]);
				if (substr($linkUrl, 0, 1) == '/') {
					$urlParts = parse_url($url);
					$faviconURL = $urlParts['scheme'].'://'.$urlParts['host'].$linkUrl;
				} elseif (substr($linkUrl, 0, 7) == 'http://') {
					$faviconURL = $linkUrl;
				} elseif (substr($url, -1, 1) == '/') {
					$faviconURL = $url.$linkUrl;
				} else {
					$faviconURL = $url.'/'.$linkUrl;
				}
			} 
			else 
			{
				// If unsuccessful, attempt to "guess" the favicon location
				$urlParts = parse_url($url);
				$faviconURL = $urlParts['scheme'].'://'.$urlParts['host'].'/favicon.ico';
			}
			
			if($this->url_validate($faviconURL))
			{
				$r = new HTTPRequest($faviconURL, 10);
				$data = $r->DownloadToString();
				cache('favicons'.DS.$this->url2filename($url).'.ico', $data, '+1 week');
				$this->output($data);
				exit();
			}

		} 

		// Last resort
		$data = file_get_contents($this->default_url);
		$this->output($data);
		exit();
	}


	/**
	 * Checks to see if file exists at address
	 * 
	 * HTTP Codes 200|30* are success, others fail
	 * 
	 * @param  string $link Address of file
	 * @return boolean Success
	 */
	function url_validate($link) 
	{
		
		$url_parts = @parse_url($link);

		if (empty($url_parts["host"]))
			return false;

		if (!empty($url_parts["path"])) {
			$documentpath = $url_parts["path"];
		} else {
			$documentpath = "/";
		}

		if (!empty($url_parts["query"]))
			$documentpath .= "?" . $url_parts["query"];

		$host = (isset($url_parts["host"])) ? $url_parts["host"] : '';
		$port = (isset($url_parts["port"])) ? $url_parts["port"] : '';
	
		if (empty($port))
			$port = "80";

		$socket = @fsockopen($host, $port, $errno, $errstr, 30);
	
		if (!$socket)
			return false;
		
		fwrite ($socket, "HEAD ".$documentpath." HTTP/1.0\r\nHost: $host\r\n\r\n");

		$http_response = fgets( $socket, 22 );

		$responses = "/(200 OK)|(30[0-9] (Moved)|(Found))/";
		if (preg_match($responses, $http_response)) {
			fclose($socket);
			return true;
		} else {
			return false;
		}

	}
	
	
	/**
	 * Creates a filename from web address
	 *
	 * Replaces non-alphanumeric characters in the hostname with underscores
	 *
	 * @param	string	$string Address
	 * @return	string	Filename
	 **/
	function url2filename($url)
	{
		$parts = parse_url($url);
		if (isset($parts['host']))
			return preg_replace("/[^a-zA-z0-9]/", '_', $parts['host']);
		else
			return '';
	}
	
	
	/**
	 * Outputs icon data to browser
	 *
	 * @param	string	$data String containing icon data
	 **/
	function output($data)
	{
		header("Content-Type: image/x-icon");
		set_time_limit(0);
		echo($data);
	}
	
	
	/**
	 * Length of string in bytes
	 *
	 * Method is MBString-extension-aware
	 *
	 * @param	string	$string String to find length of
	 * @return	integer	Length of string
	 **/
	function stringBytes($string)
	{
		$has_mbstring = extension_loaded('mbstring') ||@dl(PHP_SHLIB_PREFIX.'mbstring.'.PHP_SHLIB_SUFFIX);
		$has_mb_shadow = (int) ini_get('mbstring.func_overload');

		if ($has_mbstring && ($has_mb_shadow & 2) ) {
		   $size = mb_strlen($string,'latin1');
		} else { 
		   $size = strlen($string);
		}
		return $size;
	}
	
	
	/**
	 * beforeFilter override
	 *
	 * This is here to override any beforeFilters you may have defined in your AppController.
	 * If you want you AppController beforeFilter to fire, just remove/comment out this method
	 **/
	function beforeFilter()
	{
		// no-op
	}
}


?>